#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## update_intellique.pl for Script de mise a jour auto in /var/www/proj/my_aptget/husiana_gpg/Newscript
## 
## ######### PROJECT DESCRIPTION : ###
##
##	Ce script creer la liste des packages installes sur
## la machine ainsi que leurs versions. Cette liste est 
## stockee dans un fichier, puis compressee, puis cryptee
## avec la cle publique d'INTELLIQUE, puis signee avec 
## la cle privee du client. Ce message est ensuite envoye
## au server web.
## 
## ###################################
## 
## Made by husianycia cedric
## Login   <husiana@intellique.com>
## 
## Started on  Fri Jan 21 10:41:18 CET 2005 Fri husianycia cedric
## Last update Mon May 30 10:21:04 2005 husianycia cedric
## 
## ###################################
##
## Version :

my	$Version = "v1.01-3";

#############################################################
## Libs

use	strict;
use	GnuPG qw( :algo );		#chiffrage et signature de fichier
use	Digest::MD5 qw(md5_hex);	#cryptage de chaine
use	LWP::UserAgent;			#connexion server WEB
use	Config::IniFiles;		#Utilisation de fichier de conf
use	SendMail;			#Envoi d'email INTELLIQUE
use	Objet_Event;			#Objet d'evenement
use	Objet_Log;			#Objet de log
use	Time::HiRes qw(time);		#grande definition d'heure

#############################################################
## Variables

my	$inifile		= "/etc/storiq/update_intellique.ini";
		# fichier contenant des variables de configuration
my	$inifilemail		= "/etc/storiq/mail_info.ini";
		# fichier contenant des variables de configuration mail
my	$Intellique_name	= "Intellique Update";
		#Nom de la cle pub d'intellique
my	$dir_msg		= "/tmp/msg/";
		#nom du repertoire contenant les messages a envoyer
my	$file_liste_package	= $dir_msg.$$;	
		#nom d'un fichier unique
		#$$ contient le numero de processus
my	$home_dir		= "";
		#nom du home dir client gnupg
my	$serial_number_file	= "";
		#fichier contenant le nbr client
my	$log_file		= "&STDERR"; 
		#nom du fichier de log contenant les erreurs
my	$serial_number		= "";
		#Numero de serie du client
my	$fichier_sortie		= "";
		#Nom du fichier crypte et sygne
my	$fichier_sortie_tmp	= "";
		#Nom du fichier crypte
my	$gpg			= "";
		#nom du nouveau gpg
my	$message		= "";
		#contient le fichier de sortie pr l'envoi au server web (auma)
my	$addr_script_serv   	= "http://www.intellique.com/auto_update/comments.pl";
		# Liens du server a qui envoyer le msg.
my	$extension		= ".gz.gpg.sign";
		# extension du message recu.
my	$path_msg_recv		= "/tmp/msg_recv/";
		# repertoire ou sont stocke les msg
my	$retest			= 0;
		# Retest est un compteur comptant le nb de tentative d'envoi du message a effectuer en cas d'echec de connexion
my	$bodypresent		= 0;
		# flag pr savoir si y'a des maj a specifier ou non
my	$obj_event;
		# Objet Evenement
my	$obj_log;
		# Objet de log initialise a STDERR
my	$mail_client		= "";
		# Adresse mail du client
my	$smtp_host		= "";
		# serveur smtp

#############################################################
## Variables pour l'envoi d'email

my	%mail = ('dest_mail'	=> $mail_client, 
		 'smtp_host'	=> $smtp_host,
		 'from_mail'	=> "support\@intellique.com",
		 'from_name'	=> "StorIQ Update",
		 'dest_name'	=> "Client StorIQ Update",
		 'subject'	=> "[INTELLIQUE] Mises a jour disponibles.",
		 'mailbody'	=> "Des mises a jour de securite sont disponibles.\n");

#############################################################
## Main

sub	main()
{
    ## print "recupere les variables de conf dans $inifile\n";
    config_variables($inifile);
    config_mail($inifilemail);

    $gpg = new GnuPG (homedir => $home_dir);
  
    ## print "creation des objets\n";
    print STDERR "probleme creation Event\n" and exit unless ($obj_event = new Objet_Event());
    print STDERR "probleme creation log\n" and exit unless ($obj_log = new Objet_Log($log_file));
    
    ## print "creation du repertoire $dir_msg\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : creation du repertoire " . $dir_msg);
    mkdir ($dir_msg);
    mkdir ($path_msg_recv);
    
    ##print "creation de la liste des packages\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : creation de la liste des packages");
    create_liste_package();
    
    ## print "compresse la liste des packages\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : compresse la liste des packages");
    compression_liste_package();
    
    ## print "parse $serial_number_file\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : parse ".$serial_number_file);
    $serial_number = parse_serial_number($serial_number_file);
    $fichier_sortie = $file_liste_package . ".gpg";
    $fichier_sortie_tmp	= $file_liste_package . "NOSIGN.gpg";
    
    ## print "crypte puis je signe $fichier_sortie\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : crypte puis je signe : " . $fichier_sortie);
    crypt_sign();
    
    ## print "supprime $fichier_sortie_tmp\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : Suppresion de " . $fichier_sortie_tmp);
    unlink ($fichier_sortie_tmp);
    
    ## print "supprime $file_liste_package\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : Suppresion de " . $file_liste_package);
    unlink ($file_liste_package);
    
    ##print "formatte le message a envoyer au server WEB\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : formatte le message a envoyer au server WEB");
    $message = formate_msg();
    
    ## print "envoi le message au server WEB\n";
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : envoi le message au server WEB");
    post_serveur($message);

    $obj_log->set_event_msg($obj_event, "INFO") if ($ARGV[0] && $ARGV[0] eq "-d");
}

##################
## FONCTIONS OK ##
##################

#############################################################
## Creation body mail, liste des paquets a mettre a jour

sub body_create {
    open(MSG, $path_msg_recv.$serial_number);
    $mail{'mailbody'} .= "\nVeuillez les installer en vous rendant sur cette page :\n";
    $mail{'mailbody'} .= "\t\thttps://127.0.0.1:10000/aptget/index.cgi\n";
    $mail{'mailbody'} .= "\n\tVoici la liste ainsi que les versions :\n\n";
    my	@tmp;
    while (<MSG>){
	@tmp = split("==", $_);
	if ($tmp[2] eq "intS" || $tmp[2] eq "debS"){
	    $bodypresent = 1;
	    $mail{'mailbody'} .= $tmp[0]." => ".$tmp[1]." => ".$tmp[4];
	}
    }
    close (MSG);
}

#############################################################
## Envoi d'email

sub send_email {
      
    body_create() if (-e $path_msg_recv.$serial_number);
    ## print $mail{'mailbody'};
    SendMail::send_email(%mail) if ($bodypresent);
    $obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : OK, mail envoye!");
    ## print "OK, mail envoye!\n";
}

#############################################################
## Verification de la signature du message recu, si c'est ok
## on decrypte, decompresse et on le place ds le repertoire
## $path_msg_recv avec comme nom le num de serie!

sub verif_msg_recv
{
    my	$sig;
    my  $name = $path_msg_recv.$serial_number.$extension;
    unlink("/tmp/tmp.txt");
    eval {
	$sig = $gpg->verify(signature => $name);
	`gpg --output /tmp/tmp.txt $name 2> /dev/null`;
	$name =~ s/($extension)/.gz/;
	$gpg->decrypt(armor => 1, sign => 1, output => $name, ciphertext => "/tmp/tmp.txt",passphrase => capture_Mac_Address());
	`gunzip -f $name 2> /dev/null`;
    };
    if ( $@ ) {
	my ($variable, @tableau) = split (" at", $@);
	print_error("ERROR: verify_sign $variable");
    }
    my %usr = %$sig;
    if ($usr{user} =~ /($Intellique_name)/) {
	##print "Ok, on peut envoyer un mail pr dire qu'on a des news\n";
	$obj_event->set_event("INFO", time(), "INFO ".localtime(time())." : Ok, on peut envoyer un mail pr dire qu'on a des news");
	send_email();
    }else{
	##print "Signature du msg incorrecte!\n";
	print_error("Signature incorrecte!");
    }
    unlink("/tmp/tmp.txt");
    unlink($path_msg_recv.$serial_number.$extension);
}

#############################################################
## envoi du message sur le server web

sub post_serveur 
{
    my $message	= shift;
    my $ua	= LWP::UserAgent->new;
    my $req	= HTTP::Request->new(POST => $addr_script_serv);
    my $taille	= length($message);

    $req->content("name=$serial_number&length=$taille&message=$message");
    
    # Envoi de la requete et attente reponse
    my $res	= $ua->request($req);
    
    # Test de la reponse
    # enregistrement du message

    if ($res->is_success) {
	if ($res->content =~ /^($serial_number)/) {
	    my $recv = $res->content;
	    $recv =~ s/($serial_number)//;
	    $recv =~ tr/!/+/;
	    open(MSG, ">".$path_msg_recv.$serial_number.$extension);
	    print MSG $recv;
	    close (MSG);
	    verif_msg_recv();
	} else { 
	    if ($res->content =~ /(Error.*)/) {
		print_warn($res->content);
	    }
	}
    } else { #si on arrive pas a ce connecter on retest 3 fois au cas ou :)
	post_serveur($message) if ($retest < 3);
	$retest++;
	print_error("Error: post_server envoi msg impossible") if ($retest >= 3);
    }
}

#############################################################
## Recuperation des valeurs dans le fichier ini

sub config_variables {
    my $config_file = shift;
    
    my $cfg = new Config::IniFiles(-file => "$config_file" ) 
	or print_error("ERROR can't open ini file $config_file \n");
    
    $serial_number_file = $cfg->val("general", "serial_number_file");
    $home_dir = $cfg->val("general", "homedir");
    $log_file = $cfg->val("general", "log_file");
}

sub config_mail {
    my $config_file = shift;
    
    my $cfg = new Config::IniFiles(-file => "$config_file" ) 
	or print_error("ERROR can't open ini file $config_file \n");
    
    $mail_client = $cfg->val("general", "mail_client");
    $smtp_host = $cfg->val("general", "smtp_host");
}

#############################################################
## creation du message "envoyable"!

sub formate_msg
{
    my $message	= "";
    
    print_error("ERROR formate_msg!") if (!(-e $fichier_sortie));
    
    if (!open(FH, $fichier_sortie)) {
	print_error("ERROR formate_msg reading $fichier_sortie\n");}
    
    while (<FH>) {
	$message .= $_;
    }
    close (FH);
    
    $message =~ tr/+/!/;
    
    return $message;
}

#############################################################
## Concatenation des Mac_adress afin de crer la passphrase

sub	capture_Mac_Address {
    my $mac_eth0 = `/sbin/ifconfig eth0 2> /dev/null`;
    my $mac_eth1 = `/sbin/ifconfig eth1 2> /dev/null`;
    
    if ($mac_eth0 =~ /HWaddr (\w+:\w+:\w+:\w+:\w+:\w+)/) { $mac_eth0 = $1;}
    else { $mac_eth0 = "00:00:00:00:00:00";}
    
    if ($mac_eth1 =~ /HWaddr (\w+:\w+:\w+:\w+:\w+:\w+)/) { $mac_eth1 = $1;}
    else { $mac_eth1 = "00:00:00:00:00:00";}
    
    return (md5_hex($mac_eth0) . md5_hex($mac_eth1));
}

#############################################################
## parsing du fichier contenant le serial number

sub parse_serial_number {
    my $config = shift;
    my @settings_serial_number = ();
    my $serial_number;
    
    print_error("ERROR $config does not exist.\n") if (!(-e $config));
    
    print_error("ERROR reading $config\n") if (!open(filehandle1, $config));
    
    @settings_serial_number = <filehandle1>;
    
    close (filehandle1);
    
    for (my $i = 0; $i < ($#settings_serial_number + 1); $i++){
	$settings_serial_number[$i] = "" if ($settings_serial_number[$i] =~ /^\#/);
	if ($settings_serial_number[$i] !~ /^\s*$/) {
	    $settings_serial_number[$i] =~ s/\s*//g;
	    $serial_number = $settings_serial_number[$i];
	}
    }
    
    print_error("ERROR pas de numero de serie\n") if (!$serial_number);
    
    return $serial_number;
}

#############################################################
## Fonction qui crypte, puis signe le message

sub crypt_sign {

    eval {
	$gpg->encrypt(recipient => $Intellique_name, output => $fichier_sortie_tmp, armor => 1, plaintext => $file_liste_package);
	
	$gpg->sign(recipient => $Intellique_name, 'local-user' => $serial_number, output => $fichier_sortie, armor => 1, plaintext => $fichier_sortie_tmp, passphrase => capture_Mac_Address());
    };

    if ( $@ ) {

	my ($variable, @tableau) = split (" at", $@);
  	print_error("ERROR: crypt_sign $variable");
    }
}

#############################################################
## Compression du fichier contenant la liste des packages

sub	compression_liste_package
{
    eval {
	system("gzip $file_liste_package");
    };
    $file_liste_package .= ".gz";
    if ( $@ ){
	my ($variable, @tableau) = split (" at", $@);
	print_error("ERROR: compression_liste_package $variable");
    }
}


#############################################################
## Fonction de retour de warning ds le fichier de log

sub	print_warn
{
    $obj_event->set_event("WARN", time(), "WARN CODE ".localtime(time())." : ".$_[0]);
    $obj_log->set_event_msg($obj_event, "INFO") if ($ARGV[0] && $ARGV[0] eq "-d");
    $obj_log->set_event_msg($obj_event, "ERROR");

    $obj_log->set_event_msg($obj_event, "WARN");

    exit 1;
}

#############################################################
## Fonction de retour d'erreur ds le fichier de log

sub	print_error
{
    $obj_event->set_event("ERROR", time(), "FATAL ERROR ".localtime(time())." : ".$_[0]);
    $obj_log->set_event_msg($obj_event, "INFO") if ($ARGV[0] && $ARGV[0] eq "-d");
    $obj_log->set_event_msg($obj_event, "ERROR");

    $obj_log->set_event_msg($obj_event, "WARN");

    exit 1;
}

#############################################################
## Fonction qui creer la liste des packages ds le fichier specifie

sub	create_liste_package
{ 
    eval { 
	system('dpkg-awk "Status:.* installed$" -- Package Version | awk \'{print $2}\' | egrep -v \'^$ \'| xargs -n2 echo > ' . $file_liste_package . ' | perl -pe \'s/(\S+)\s(\S+)\s(\S+)/$2 $1 $3/\' | sort -g ');
    };
    if ( $@ )
    {
	my ($variable, @tableau) = split (" at", $@);
	print_error("ERROR: create_liste_package $variable");
    }
}

###################
## /FONCTIONS OK ##
###################

#############################################################
## Appel du main

main();
exit 1;
